'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.processTemplate = processTemplate;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Processes a template file and replaces variables with values.
 * @param {string} templatePath - The path to the template file
 * @param {string} destinationPath - The path to the destination file
 * @param {object} options - The options to replace in the template
 * @return {Promise<void>} nothing
 */
async function processTemplate(_ref) {
	var templatePath = _ref.templatePath,
	    destinationPath = _ref.destinationPath,
	    options = _ref.options;

	// open the template file
	var file = await _fs.promises.readFile(templatePath, 'utf8');

	for (var key in options) {
		var value = options[key];
		var regex = new RegExp('<%-\\s*' + key + '\\s*%>', 'g');
		file = file.replace(regex, value);
	}

	await _fsExtra2.default.ensureDir(_path2.default.dirname(destinationPath));
	await _fs.promises.writeFile(destinationPath, file);
}